//////////////////////////////////////////////////////////////////
// Fir filter example.
//
// Written for Generic Platform Support Package.
//
// Copyright(c) 2003-2009 Impulse Accelerated Technologies, Inc.
//
// fir_sw.c: includes the software test bench processes and
// main() function.
//
#ifdef WIN32
#include <windows.h>
#include <sys/types.h>
#include <sys/stat.h>
#else  // ! WIN32
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#endif
#include <stdio.h>
#include "co.h"
#include "co_math.h"
#include "cosim_log.h"
#include "fir.h"

extern co_architecture co_initialize(void *);

void test_producer(co_stream waveform_in)
{

	int c, jj, ii;
	cosim_logwindow log = cosim_logwindow_create("test_producer");

	// Read waveform and coefficients from a file
	const char * CoefFileName = COEF_FILE;   	// See fir.h
	const char * FileName = INPUT_FILE;   		// See fir.h
	FILE * inFile, * coefFile;
	picobusDataType coefValue		[128 / picobusWidth];
	picobusDataType waveformValue	[128 / picobusWidth];
	int coefcount = 0;
	BOOL done = FALSE;

	coefFile = fopen(CoefFileName, "r");
	if ( coefFile == NULL ) {
		fprintf(stderr, "Error opening coefficients file %s\n", CoefFileName);
		c = getc(stdin);
		exit(-1);
	}
	// Read and write the coefficient data...
	co_stream_open(waveform_in, O_WRONLY, UINT_TYPE(picobusWidth));
	cosim_logwindow_write(log, "Sending coefficients...\n");

	for (ii = 0; ii < sizeof(coefValue); ii++)
		((char *)coefValue)[ii] = 0;

	for (coefcount = 0; coefcount < TAPS; coefcount+=jj)
	{
		for (jj = 0; jj < (128 / picobusWidth); jj++)
		{
			if (fscanf(coefFile,"%d",(picobusDataType * )(&coefValue[jj])) < 1)
			{
				printf("WARNING: coefficients file did not have data elements equal to requested TAPS count\r\n");
				coefcount = TAPS;//hitting this line indicates a potential problem; is the coeficients file long enough?
				break;
			}
			cosim_logwindow_fwrite(log, "Value: %x\n", coefValue[jj]);
		}
		co_stream_write(waveform_in, &coefValue[0], jj * sizeof(picobusDataType));
		memset(coefValue, 0, sizeof(coefValue));
	}

	cosim_logwindow_fwrite(log, "Finished writing %d coefficients.\n", coefcount);
	fclose(coefFile);

	inFile = fopen(FileName, "r");
	if ( inFile == NULL ) {
		fprintf(stderr, "Error opening waveform file %s\n", FileName);
		c = getc(stdin);
		exit(-1);
	}

	// Now read and write the waveform data...
	co_stream_open(waveform_in, O_WRONLY, UINT_TYPE(picobusWidth));
	cosim_logwindow_write(log, "Sending waveform...\n");

	for (ii = 0; ii < sizeof(waveformValue); ii++)
		((char *)waveformValue)[ii] = 0;
	for (done = FALSE; done == FALSE;)
	{
		for (jj = 0; jj < (128 / picobusWidth); jj++)
		{
			if (fscanf(inFile,"%d",(uint32 * )(&(waveformValue[jj]))) < 1)
			{
				done = TRUE;
				break;
			}
			cosim_logwindow_fwrite(log, "Value: %x\n", waveformValue[jj]);
		}
		co_stream_write(waveform_in, &waveformValue[0], jj * sizeof(picobusDataType));
		memset(waveformValue, 0, sizeof(waveformValue));
		cosim_logwindow_fwrite(log, "Value: %x\n", waveformValue);
	}

	cosim_logwindow_write(log, "Finished writing waveform.\n");
	co_stream_close(waveform_in);
	fclose(inFile);
}

void test_consumer(co_stream waveform_out)
{
	int ii;
	picobusDataType waveformValue[4];
	unsigned int waveformCount = 0;
  	const char * FileName = OUTPUT_FILE;   // See fir.h
	FILE * outFile;

	cosim_logwindow log = cosim_logwindow_create("test_consumer");

	outFile = fopen(FileName, "w");
	if ( outFile == NULL ) {
		fprintf(stderr, "Error opening waveform file %s for writing\n", FileName);
		exit(-1);
	}

	co_stream_open(waveform_out, O_RDONLY, UINT_TYPE(picobusWidth));

	cosim_logwindow_write(log, "Consumer reading data...\n");
	while ( co_stream_read(waveform_out, &waveformValue[0], 128/8) == co_err_none )
	{
		for (ii = 0; ii < 128/picobusWidth; ii++)
		{
			fprintf(outFile, "%d\n", waveformValue[ii]);
			cosim_logwindow_fwrite(log, "Value: %x\n", waveformValue[ii]);
			printf("Filtered value: %x\n", waveformValue[ii]);
			waveformCount++;
		}
		memset(waveformValue, 0, sizeof(waveformValue));
	}
	cosim_logwindow_fwrite(log, "Consumer read %d waveform datapoints...\n", waveformCount);
	co_stream_close(waveform_out);
	fclose(outFile);
}

int main(int argc, char *argv[])
{
	co_architecture my_arch = NULL;
	int c;
	FILE * fileNameTest;
	IF_NSIM(
		PicoImpulseParameters picoParams;//only relevant for generated software
		picoParams.model = PICO_M501LX;//will only open a channel to a Pico M501
		picoParams.cardNumber = -1;//unspecified
		 //the default name for the bit file generated by the hardware build step.
		picoParams.bitFileNameP = "c:\\Impulse\\trunk\\Examples\\M5XX\Fir12\\export_hw\\m501_lx240t_toplevel.bit";

		fileNameTest = fopen(picoParams.bitFileNameP, "r");
		if (!fileNameTest) {printf("Error, bit file \"%s \" not found", picoParams.bitFileNameP); return 0;}
		else fclose(fileNameTest);
	)

	printf("Impulse C is Copyright 2003-2009 Impulse Accelerated Technologies, Inc.\n");
	IF_SIM( my_arch = co_initialize(NULL);
			co_execute(my_arch);)


	IF_NSIM(co_initialize((void *)&picoParams);
			co_execute(NULL);
			)
	printf("\n\nApplication complete. Press the Enter key to continue.\n");
	c = getc(stdin);
	return(0);
}
