//////////////////////////////////////////////////////////////////
// Passthrough filter example.
//
// Copyright(c) 2003-2009 Impulse Accelerated Technologies, Inc.
//
// passthrough_hw.c: includes the hardware process and configuration
// function.
//
// See additional comments in passthrough.h.
//

#include <stdio.h>
#include "co.h"
#include "cosim_log.h"
#include "passthrough.h"

extern void test_producer(co_stream waveform_in);
extern void test_consumer(co_stream waveform_out);

//////////////////////////////////////////////////
// This is the
// passthrough hardware process.
// The process accepts a stream representing a
// waveform of 32-bit samples and returns the
// stream
//
// Adjust the StageDelay pragma to balance max
// frequency with pipeline rate
//
// This sample has been modified to use the Pico M501's
// 128-bit bus.

void passthrough(co_stream filter_in, co_stream filter_out)
{
	IF_SIM(int samplesread = 0;)
	IF_SIM(int sampleswritten = 0;)

	picobusDataType nSample;

	IF_SIM( cosim_logwindow log; )
	IF_SIM ( log = cosim_logwindow_create("passthrough"); )

    do {	// Hardware processes run forever

		co_stream_open(filter_in, O_RDONLY,  UINT_TYPE((picobusWidth)));
		co_stream_open(filter_out, O_WRONLY, UINT_TYPE((picobusWidth)));

		while (1) {
			co_stream_read(filter_in, &nSample, sizeof(picobusDataType));
			IF_SIM(samplesread++;)
			co_stream_write(filter_out, &nSample, sizeof(picobusDataType));
			IF_SIM(sampleswritten++;)
			IF_SIM(if (sampleswritten == 10) break;)
		}

		co_stream_close(filter_in);
		co_stream_close(filter_out);

		IF_SIM(break;)	// Only run once for desktop simulation

	} while(1);
}

//
// Impulse C configuration function
//
void config_passthrough(void *arg)
{
	co_stream waveform_in;
	co_stream waveform_out;
	co_process passthrough_process;
	co_process producer_process;
	co_process consumer_process;
	IF_SIM(cosim_logwindow_init();)

	waveform_in = co_stream_create("waveform_in", UINT_TYPE((picobusWidth)), BUFSIZE);
	waveform_out = co_stream_create("waveform_out", UINT_TYPE((picobusWidth)), BUFSIZE);

	producer_process = co_process_create("producer_process", (co_function)test_producer,
                                         1, waveform_in);
	passthrough_process = co_process_create("passthrough_process", (co_function)passthrough,
                                    2, waveform_in, waveform_out);
	consumer_process = co_process_create("consumer_process",(co_function)test_consumer,
                                         1, waveform_out);
	// Assign passthrough process to hardware elements
	co_process_config(passthrough_process, co_loc, "PE0");
}

co_architecture co_initialize(int param)
{
	return(co_architecture_create("passthrough_arch","Generic_VHDL",config_passthrough,(void *)param));
}

